#!/usr/bin/perl -w

package ALTLinux::Girar::UploadMethod;
use strict;
use warnings;
use File::Path qw(make_path);
use File::Basename;
use KyotoCabinet;
use IPC::Run3;

use constant UNKNOWN => 0;
use constant SRPM => 1;
use constant GIT => 2;


our $VERSION = 0.01;

#our $default_expire = 5/24; # -M; 5/24=5 hours
our $default_branch = 'sisyphus';
our $default_verbose = 0;
our $girar_url="git://git.altlinux.org";


my @cachedirs=grep {$_} ($ENV{HOME}.'/.cache/altlinux-girar','.');
my $dbfile=$cachedirs[0].'/upload_method.kch';

my $db = new KyotoCabinet::DB;



make_path($cachedirs[0]);
# open the database
if (!$db->open($dbfile, $db->OWRITER | $db->OCREATE)) {
    printf STDERR ("ALTLinux::Girar::UploadMethod: $dbfile: open error: %s\n", $db->error);
    unlink $dbfile;
    $db=undef unless $db->open($dbfile, $db->OWRITER | $db->OCREATE);
}

END {
    if ($db and !$db->close) {
	printf STDERR ("ALTLinux::Girar::UploadMethod: $dbfile: close error: %s\n", $db->error);
    }
}

sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
#	-expire=> $default_expire,
	-branch=> $default_branch,
	-verbose=> $default_verbose,
	@_,
    };
    bless($self,$class);
    return $self;
}
 
sub get {
    my ($self,$name,$version,$release,$branch)=@_;
    $branch=$self->{-branch} unless $branch;
    my $key=join("\0",$name,$version,$release,$branch);
    my $value;
    $value = $db->get($key) if $db;
    return $value if defined $value;
    $value=&method($name,$branch);
    if ($db and not $db->set($key, $value)) {
	printf STDERR ("ALTLinux::Girar::UploadMethod: $dbfile: internal set error: %s\n", $db->error);
    }
    return $value;
}

sub method {
    my ($name,$repobranch)=@_;
    $repobranch||=$default_branch;
    my $gitrepo=substr($name,0,1).'/'.$name.'.git';
    my $gitpath="refs/heads/$repobranch";

    my ($gitcommit, $srpmcommit);
    my $retval=UNKNOWN;
    run3 [qw/git ls-remote -h/, "$girar_url/gears/$gitrepo", $gitpath], \undef, \$gitcommit, \undef;
    my $GITRETVAL=$?;

    if ($GITRETVAL == 0 and $gitcommit) {
	$retval=GIT;
	return $retval unless wantarray;
    }

    run3 [qw/git ls-remote -h/, "$girar_url/srpms/$gitrepo", $gitpath], \undef, \$srpmcommit, \undef;
    my $SRPMRETVAL=$?;

    $retval=SRPM if ($retval==UNKNOWN and $SRPMRETVAL==0 and $srpmcommit);
    unless (wantarray) {
	return $retval;
    } else {
	my $has_git_history= $GITRETVAL==0;
	my $has_srpm_history= $SRPMRETVAL==0;
	$gitcommit=~s/\s.*$//s;
	$srpmcommit=~s/\s.*$//s;
	return $retval, $has_git_history, $has_srpm_history, $gitcommit, $srpmcommit;
    }
}




1;

=head1	NAME

ALTLinux:: - ALTLinux Girar upload method query library.

=head1	DESCRIPTION

ALTLinux Girar upload method query library provides an interface
to query and cache package's ALTLinux Girar upload method
(using srpm or git repository).
Upload methods are cached by the name/version/release/branch signature.

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	COPYING

Copyright (c) 2010 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

